{% extends 'admin/change_list.html' %}

{% block object-tools %}
    <div>
        <form action="fetch_tenantusers/" method="POST">
            {% csrf_token %}
                <button type="submit" style="cursor:pointer;border-radius: 5px;height: 30px;">Fetch Tenant Users</button>
        </form>
    </div>
    <br />
    {{ block.super }}
{% endblock %}
