# Generated by Django 4.2.9 on 2024-02-20 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbintegrator_office365", "0002_remove_calendar_owner_remove_calendarevent_activity_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="event",
            options={
                "permissions": [("administrate_event", "Can Administrate call and calendar events")],
                "verbose_name": "Event",
                "verbose_name_plural": "Events",
            },
        ),
    ]
