# Generated by Django 4.1.7 on 2023-04-21 10:50

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "wbintegrator_office365",
            "0001_initial_squashed_squashed_0003_alter_calendar_owner_alter_calendarevent_organizer_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="calendar",
            name="owner",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="activity",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="calendar",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="event",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="organizer",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="participants",
        ),
        migrations.RemoveField(
            model_name="calendarevent",
            name="recurrence",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="activity",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="calendar_event",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="organizer",
        ),
        migrations.RemoveField(
            model_name="calendareventinstance",
            name="participants",
        ),
        migrations.RemoveField(
            model_name="location",
            name="address",
        ),
        migrations.RemoveField(
            model_name="tenantuser",
            name="enable_synchronization_activity",
        ),
        migrations.DeleteModel(
            name="Address",
        ),
        migrations.DeleteModel(
            name="Calendar",
        ),
        migrations.DeleteModel(
            name="CalendarEvent",
        ),
        migrations.DeleteModel(
            name="CalendarEventInstance",
        ),
        migrations.DeleteModel(
            name="Location",
        ),
        migrations.DeleteModel(
            name="RecurrencePattern",
        ),
    ]
